import { AuthModeType } from '@algolia/client-common';
import { CreateClient } from '@algolia/client-common';
import { RequestOptions } from '@algolia/transporter';
import { Transporter } from '@algolia/transporter';
import { TransporterOptions } from '@algolia/transporter';
import { WaitablePromise } from '@algolia/client-common';

export declare const addApiKey: (base: SearchClient) => (acl: readonly string[], requestOptions?: (AddApiKeyOptions & Pick<RequestOptions, string | number>) | undefined) => Readonly<WaitablePromise<AddApiKeyResponse>>;

export declare type AddApiKeyOptions = {
    /**
     * A Unix timestamp used to define the expiration date of the API key.
     */
    readonly validity?: number;
    /**
     * Specify the maximum number of hits this API key can retrieve in one call.
     * This parameter can be used to protect you from attempts at retrieving your entire index contents by massively querying the index.
     */
    readonly maxHitsPerQuery?: number;
    /**
     * Specify the maximum number of API calls allowed from an IP address per hour. Each time an API call is performed with this key, a check is performed.
     */
    readonly maxQueriesPerIPPerHour?: number;
    /**
     * Specify the list of targeted indices. You can target all indices starting with a prefix or ending with a suffix using the ‘*’ character.
     */
    readonly indexes?: readonly string[];
    /**
     * Specify the list of referers. You can target all referers starting with a prefix, ending with a suffix using the ‘*’ character.
     */
    readonly referers?: readonly string[];
    /**
     * Specify the list of query parameters. You can force the query parameters for a query using the url string format.
     */
    readonly queryParameters?: string;
    /**
     * Specify a description of the API key. Used for informative purposes only. It has impact on the functionality of the API key.
     */
    readonly description?: string;
};

export declare type AddApiKeyResponse = {
    /**
     * The returned api key.
     */
    readonly key: string;
    /**
     * Date of creation of the api key.
     */
    readonly createdAt: string;
};

export declare const assignUserID: (base: SearchClient) => (userID: string, clusterName: string, requestOptions?: RequestOptions | undefined) => Readonly<Promise<AssignUserIDResponse>>;

export declare type AssignUserIDResponse = {
    /**
     * Date of creation of the userId.
     */
    readonly createdAt: string;
};

export declare const assignUserIDs: (base: SearchClient) => (userIDs: readonly string[], clusterName: string, requestOptions?: RequestOptions | undefined) => Readonly<Promise<AssignUserIDsResponse>>;

export declare type AssignUserIDsResponse = {
    /**
     * Date of creation of the userId
     */
    readonly createdAt: string;
};

export declare type AutomaticFacetFilter = {
    /** Attribute to filter on. This must match a facet placeholder in the rule’s pattern. */
    readonly facet?: string;
    /** Whether the filter is disjunctive (true) or conjunctive (false). */
    readonly disjunctive?: boolean;
    /** Score for the filter. Typically used for optional or disjunctive filters. */
    readonly score?: number;
};

export declare const batch: (base: SearchIndex) => (requests: readonly BatchRequest[], requestOptions?: RequestOptions | undefined) => Readonly<WaitablePromise<BatchResponse>>;

export declare const BatchActionEnum: {
    readonly [key: string]: BatchActionType;
};

export declare type BatchActionType = 'addObject' | 'updateObject' | 'partialUpdateObject' | 'partialUpdateObjectNoCreate' | 'deleteObject';

export declare type BatchRequest = {
    readonly action: BatchActionType;
    readonly body: object;
};

export declare type BatchResponse = {
    readonly taskID: number;
    readonly objectIDs: readonly string[];
};

export declare const browseObjects: (base: SearchIndex) => <TObject extends ObjectWithObjectID>(requestOptions?: (SearchOptions & BrowseOptions<TObject> & RequestOptions) | undefined) => Readonly<Promise<void>>;

export declare type BrowseOptions<TObject> = {
    readonly batch?: (batch: readonly TObject[]) => any;
    readonly hitsPerPage?: number;
};

export declare type BrowseResponse<TObject> = {
    readonly hits: readonly TObject[];
    readonly cursor?: string;
};

export declare const browseRules: (base: SearchIndex) => (requestOptions?: (SearchRulesOptions & BrowseOptions<Rule> & RequestOptions) | undefined) => Readonly<Promise<void>>;

export declare const browseSynonyms: (base: SearchIndex) => (requestOptions?: (SearchSynonymsOptions & BrowseOptions<Synonym> & RequestOptions) | undefined) => Readonly<Promise<void>>;

export declare const chunk: (base: SearchIndex) => (bodies: readonly object[], action: BatchActionType, requestOptions?: (RequestOptions & ChunkOptions) | undefined) => Readonly<WaitablePromise<readonly BatchResponse[]>>;

/**
 * @internal
 */
export declare type ChunkOptions = {
    readonly batchSize?: number;
};

export declare const clearObjects: (base: SearchIndex) => (requestOptions?: RequestOptions | undefined) => Readonly<WaitablePromise<DeleteResponse>>;

export declare const clearRules: (base: SearchIndex) => (requestOptions?: RequestOptions | undefined) => Readonly<WaitablePromise<DeleteResponse>>;

export declare const clearSynonyms: (base: SearchIndex) => (requestOptions?: (ClearSynonymsOptions & RequestOptions) | undefined) => Readonly<WaitablePromise<DeleteResponse>>;

export declare type ClearSynonymsOptions = {
    readonly forwardToReplicas?: boolean;
};

export declare type Cluster = {
    /**
     * The cluster name
     */
    readonly clusterName: string;
    /**
     * Number of records in the cluster.
     */
    readonly nbRecords: number;
    /**
     * Number of users assign to the cluster.
     */
    readonly nbUserIDs: number;
    /**
     * Data size taken by all the users assigned to the cluster.
     */
    readonly dataSize: number;
};

export declare type Condition = {
    /** Query patterns are expressed as a string with a specific syntax. A pattern is a sequence of tokens. */
    readonly pattern?: string;
    /** { is | startsWith | endsWith | contains }: Whether the pattern must match the beginning or the end of the query string, or both, or none. */
    readonly anchoring?: 'is' | 'startsWith' | 'endsWith' | 'contains';
    /**
     * Rule context (format: [A-Za-z0-9_-]+). When specified, the rule is contextual and applies only when the same context is specified at query time (using the ruleContexts parameter).
     * When absent, the rule is generic and always applies (provided that its other conditions are met, of course).
     */
    readonly context?: string;
    /**
     * If set to true, alternatives make the rule to trigger on synonyms, typos and plurals.
     * Note that setting ignorePlurals to false overrides this parameter.
     */
    readonly alternatives?: boolean;
};

export declare type Consequence = {
    /**
     * Additional search parameters. Any valid search parameter is allowed.
     */
    readonly params?: ConsequenceParams & Pick<SearchOptions, Exclude<keyof SearchOptions, 'query'>>;
    /**
     * Objects to promote as hits.
     */
    readonly promote?: readonly ConsequencePromote[];
    /**
     * Objects to hide from hits.
     */
    readonly hide?: ReadonlyArray<{
        readonly objectID?: string;
    }>;
    /**
     * Custom JSON object that will be appended to the userData array in the response.
     * This object is not interpreted by the API. It is limited to 1kB of minified JSON.
     */
    readonly userData?: any;
};

export declare type ConsequenceParams = {
    /**
     * When providing a string, it replaces the entire query string.
     * When providing an object, it describes incremental edits to be made to the query string (but you can’t do both).
     */
    readonly query?: ConsequenceQuery | string | ReadonlyMap<string, readonly string[]>;
    /** Names of facets to which automatic filtering must be applied; they must match the facet name of a facet value placeholder in the query pattern. */
    readonly automaticFacetFilters?: readonly AutomaticFacetFilter[] | readonly string[];
    /**
     * Same syntax as automaticFacetFilters, but the engine treats the filters as optional.
     * Behaves like optionalFilters.
     */
    readonly automaticOptionalFacetFilters?: readonly AutomaticFacetFilter[];
};

export declare type ConsequencePromote = {
    /** Unique identifier of the object to promote. */
    readonly objectID?: string;
    /** Promoted rank for the object (zero-based). */
    readonly position?: number;
};

export declare type ConsequenceQuery = {
    /** List of edits */
    readonly edits?: ReadonlyArray<{
        /**
         * Type of edit @see EditType
         */
        readonly type?: string;
        /** Text or patterns to remove from the query string. */
        readonly delete?: string;
        /** Text that should be inserted in place of the removed text inside the query string. */
        readonly insert?: string;
    }>;
};

export declare const copyIndex: (base: SearchClient) => (from: string, to: string, requestOptions?: (CopyIndexOptions & RequestOptions) | undefined) => Readonly<WaitablePromise<IndexOperationResponse>>;

export declare type CopyIndexOptions = {
    readonly scope?: readonly ScopeType[];
};

export declare const copySettings: (base: SearchClient) => (from: string, to: string, requestOptions?: RequestOptions | undefined) => Readonly<WaitablePromise<IndexOperationResponse>>;

export declare const copySynonyms: (base: SearchClient) => (from: string, to: string, requestOptions?: RequestOptions | undefined) => Readonly<WaitablePromise<IndexOperationResponse>>;

export declare function createBrowsablePromise<TObject>(options: {
    readonly shouldStop: (response: BrowseResponse<TObject>) => boolean;
    readonly request: (data: {
        readonly page: number;
    }) => Readonly<Promise<BrowseResponse<TObject>>>;
} & BrowseOptions<TObject>): Readonly<Promise<void>>;

export declare type CreateIndex = <TMethods extends {
    readonly [key: string]: (base: SearchIndex) => (...args: any) => any;
}>(indexName: string, options?: {
    readonly methods?: TMethods;
}) => SearchIndex & {
    [key in keyof TMethods extends string ? keyof TMethods : never]: ReturnType<TMethods[key]>;
};

export declare function createMissingObjectIDError(): Error;

export declare function createObjectNotFoundError(): Error;

export declare const createSearchClient: CreateClient<SearchClient, SearchClientOptions & TransporterOptions>;

export declare function createValidUntilNotFoundError(): Error;

export declare const deleteApiKey: (base: SearchClient) => (apiKey: string, requestOptions?: RequestOptions | undefined) => Readonly<WaitablePromise<DeleteApiKeyResponse>>;

export declare type DeleteApiKeyResponse = {
    readonly deletedAt: string;
};

export declare const deleteBy: (base: SearchIndex) => (filters: DeleteByFiltersOptions, requestOptions?: RequestOptions | undefined) => Readonly<WaitablePromise<DeleteResponse>>;

export declare type DeleteByFiltersOptions = {
    readonly filters?: string;
    readonly facetFilters?: ReadonlyArray<readonly string[]>;
    readonly numericFilters?: ReadonlyArray<readonly string[]>;
    readonly tagFilters?: readonly string[] | ReadonlyArray<readonly string[]>;
    readonly aroundLatLng?: string;
    readonly aroundLatLngViaIP?: boolean;
    readonly insideBoundingBox?: ReadonlyArray<readonly number[]>;
    readonly insidePolygon?: ReadonlyArray<readonly number[]>;
};

export declare const deleteIndex: (base: SearchIndex) => (requestOptions?: RequestOptions | undefined) => Readonly<WaitablePromise<DeleteResponse>>;

export declare const deleteObject: (base: SearchIndex) => (objectID: string, requestOptions?: RequestOptions | undefined) => Readonly<WaitablePromise<DeleteResponse>>;

export declare const deleteObjects: (base: SearchIndex) => (objectIDs: readonly string[], requestOptions?: (RequestOptions & ChunkOptions) | undefined) => Readonly<WaitablePromise<readonly BatchResponse[]>>;

export declare type DeleteResponse = {
    readonly taskID: number;
};

export declare const deleteRule: (base: SearchIndex) => (objectID: string, requestOptions?: RequestOptions | undefined) => Readonly<WaitablePromise<DeleteResponse>>;

export declare const deleteSynonym: (base: SearchIndex) => (objectID: string, requestOptions?: (DeleteSynonymOptions & RequestOptions) | undefined) => Readonly<WaitablePromise<DeleteResponse>>;

export declare type DeleteSynonymOptions = {
    readonly forwardToReplicas?: boolean;
};

export declare const EditEnum: {
    readonly [key: string]: EditType;
};

export declare type EditType = 'remove' | 'replace';

export declare type EventScoring = {
    readonly type: string;
    readonly score: number;
};

export declare const exists: (base: SearchIndex) => (requestOptions?: RequestOptions | undefined) => Readonly<Promise<boolean>>;

export declare type FacetHit = {
    readonly value: string;
    readonly highlighted: string;
    readonly count: number;
};

export declare type FacetScoring = {
    readonly score: number;
};

export declare const findObject: (base: SearchIndex) => <TObject>(callback: (object: TObject & ObjectWithObjectID) => boolean, requestOptions?: (FindObjectOptions & RequestOptions) | undefined) => Readonly<Promise<FindObjectResponse<TObject>>>;

export declare type FindObjectOptions = {
    readonly paginate?: boolean;
    readonly query?: string;
};

export declare type FindObjectResponse<TObject> = {
    readonly object: TObject & ObjectWithObjectID;
    readonly position: number;
    readonly page: number;
};

export declare const generateSecuredApiKey: () => (parentApiKey: string, restrictions: SecuredApiKeyRestrictions) => string;

export declare const getApiKey: (base: SearchClient) => (apiKey: string, requestOptions?: RequestOptions | undefined) => Readonly<Promise<GetApiKeyResponse>>;

export declare type GetApiKeyResponse = {
    /**
     * A Unix timestamp used to define the expiration date of the API key.
     */
    readonly value: string;
    /**
     * Date of creation.
     */
    readonly createdAt: string;
    /**
     * List of permissions the key contains.
     */
    readonly acl: readonly string[];
    /**
     * A Unix timestamp used to define the expiration date of the API key.
     */
    readonly validity: number;
    /**
     * Specify the maximum number of hits this API key can retrieve in one call.
     * This parameter can be used to protect you from attempts at retrieving your entire index contents by massively querying the index.
     */
    readonly maxHitsPerQuery?: number;
    /**
     * Specify the maximum number of API calls allowed from an IP address per hour. Each time an API call is performed with this key, a check is performed.
     */
    readonly maxQueriesPerIPPerHour?: number;
    /**
     * Specify the list of targeted indices. You can target all indices starting with a prefix or ending with a suffix using the ‘*’ character.
     */
    readonly indexes?: readonly string[];
    /**
     * Specify the list of referers. You can target all referers starting with a prefix, ending with a suffix using the ‘*’ character.
     */
    readonly referers?: readonly string[];
    /**
     * Specify the list of query parameters. You can force the query parameters for a query using the url string format.
     */
    readonly queryParameters?: string;
    /**
     * Specify a description of the API key. Used for informative purposes only. It has impact on the functionality of the API key.
     */
    readonly description?: string;
};

export declare const getLogs: (base: SearchClient) => (requestOptions?: RequestOptions | undefined) => Readonly<Promise<GetLogsResponse>>;

export declare type GetLogsOptions = {
    readonly offset: number;
    readonly length: number;
};

export declare type GetLogsResponse = {
    /** List of logs */
    readonly logs: readonly Log[];
};

export declare const getObject: (base: SearchIndex) => <TObject>(objectID: string, requestOptions?: RequestOptions | undefined) => Readonly<Promise<TObject>>;

export declare const getObjectPosition: () => (searchResponse: SearchResponse<{}>, objectID: string) => number;

export declare const getObjects: (base: SearchIndex) => <TObject>(objectIDs: readonly string[], requestOptions?: (RequestOptions & GetObjectsOptions) | undefined) => Readonly<Promise<GetObjectsResponse<TObject>>>;

export declare type GetObjectsOptions = {
    readonly attributesToRetrieve?: readonly string[];
};

export declare type GetObjectsResponse<TObject> = {
    readonly results: readonly TObject[];
};

export declare const getPersonalizationStrategy: (base: SearchClient) => (requestOptions?: RequestOptions | undefined) => Readonly<Promise<GetPersonalizationStrategyResponse>>;

export declare type GetPersonalizationStrategyResponse = {
    readonly eventsScoring: {
        readonly [key: string]: EventScoring;
    };
    readonly facetsScoring: {
        readonly [key: string]: FacetScoring;
    };
    readonly taskID: number;
};

export declare const getRule: (base: SearchIndex) => (objectID: string, requestOptions?: RequestOptions | undefined) => Readonly<Promise<Rule>>;

export declare const getSecuredApiKeyRemainingValidity: () => (securedApiKey: string) => number;

export declare const getSettings: (base: SearchIndex) => (requestOptions?: RequestOptions | undefined) => Readonly<Promise<IndexSettings>>;

export declare const getSynonym: (base: SearchIndex) => (objectID: string, requestOptions?: RequestOptions | undefined) => Readonly<Promise<Synonym>>;

export declare const getTask: (base: SearchIndex) => (taskID: number, requestOptions?: RequestOptions | undefined) => Readonly<Promise<TaskStatusResponse>>;

export declare const getTopUserIDs: (base: SearchClient) => (requestOptions?: RequestOptions | undefined) => Readonly<Promise<GetTopUserIDsResponse>>;

export declare type GetTopUserIDsResponse = {
    /**
     * Mapping of cluster names to top users.
     */
    readonly topUsers: {
        readonly [key: string]: readonly UserIDResponse[];
    };
};

export declare const getUserID: (base: SearchClient) => (userID: string, requestOptions?: RequestOptions | undefined) => Readonly<Promise<UserIDResponse>>;

export declare type IndexOperationResponse = {
    readonly taskID: number;
};

export declare type IndexSettings = {
    readonly searchableAttributes?: readonly string[];
    readonly attributesForFaceting?: readonly string[];
    readonly unretrievableAttributes?: readonly string[];
    readonly attributesToRetrieve?: readonly string[];
    readonly ranking?: readonly string[];
    readonly customRanking?: readonly string[];
    readonly replicas?: readonly string[];
    readonly primary?: string;
    readonly maxValuesPerFacet?: number | null;
    readonly sortFacetValuesBy?: string;
    readonly attributesToHighlight?: readonly string[];
    readonly attributesToSnippet?: readonly string[];
    readonly highlightPreTag?: string;
    readonly highlightPostTag?: string;
    readonly snippetEllipsisText?: string;
    readonly restrictHighlightAndSnippetArrays?: boolean | null;
    readonly hitsPerPage?: number | null;
    readonly paginationLimitedTo?: number | null;
    readonly minWordSizefor1Typo?: number | null;
    readonly minWordSizefor2Typos?: number | null;
    readonly typoTolerance?: string | boolean;
    readonly allowTyposOnNumericTokens?: boolean | null;
    readonly disableTypoToleranceOnAttributes?: readonly string[];
    readonly disableTypoToleranceOnWords?: readonly string[];
    readonly separatorsToIndex?: string;
    readonly ignorePlurals?: readonly string[] | boolean;
    readonly queryLanguages?: readonly string[];
    readonly enableRules?: boolean | null;
    readonly queryType?: string;
    readonly removeWordsIfNoResults?: string;
    readonly advancedSyntax?: boolean | null;
    readonly advancedSyntaxFeatures?: readonly string[];
    readonly optionalWords?: readonly string[];
    readonly disablePrefixOnAttributes?: readonly string[];
    readonly disableExactOnAttributes?: readonly string[];
    readonly exactOnSingleWordQuery?: string;
    readonly alternativesAsExact?: readonly string[];
    readonly removeStopWords?: readonly string[] | boolean;
    readonly numericAttributesForFiltering?: readonly string[];
    readonly allowCompressionOfIntegerArray?: boolean | null;
    readonly attributeForDistinct?: string;
    readonly distinct?: number | boolean;
    readonly replaceSynonymsInHighlight?: boolean | null;
    readonly attributeCriteriaComputedByMinProximity?: boolean | null;
    readonly minProximity?: number | null;
    readonly responseFields?: readonly string[];
    readonly maxFacetHits?: number | null;
    readonly camelCaseAttributes?: readonly string[];
    readonly decompoundedAttributes?: {
        readonly [key: string]: readonly string[];
    };
    readonly keepDiacriticsOnCharacters?: string;
    readonly userData?: any;
};

export declare type Indice = {
    /** Index name. */
    readonly name: string;
    /** Index creation date. */
    readonly createdAt: number;
    /** Date of last update. */
    readonly updatedAt: number;
    /** Number of records contained in the index */
    readonly entries: number;
    /** Number of bytes of the index in minified format. */
    readonly dataSize: number;
    /** Number of bytes of the index binary file. */
    readonly fileSize: number;
    /** Last build time in seconds. */
    readonly lastBuildTimes: number;
    /** Number of pending indexing operations. */
    readonly numberOfPendingTasks: number;
    /** A boolean which says whether the index has pending tasks. */
    readonly pendingTask: boolean;
};

export declare const initIndex: (base: SearchClient) => CreateIndex;

export declare const listApiKeys: (base: SearchClient) => (requestOptions?: RequestOptions | undefined) => Readonly<Promise<ListApiKeysResponse>>;

export declare type ListApiKeysResponse = {
    /**
     * List of keys
     */
    readonly keys: readonly GetApiKeyResponse[];
};

export declare const listClusters: (base: SearchClient) => (requestOptions?: RequestOptions | undefined) => Readonly<Promise<ListClustersResponse>>;

export declare type ListClustersResponse = {
    /** List of clusters */
    readonly clusters: readonly Cluster[];
};

export declare const listIndices: (base: SearchClient) => (requestOptions?: RequestOptions | undefined) => Readonly<Promise<ListIndicesResponse>>;

export declare type ListIndicesResponse = {
    /** Number of pages */
    readonly nbPages: number;
    /** List of index response */
    readonly items: readonly Indice[];
};

export declare const listUserIDs: (base: SearchClient) => (requestOptions?: (ListUserIDsOptions & RequestOptions) | undefined) => Readonly<Promise<ListUserIDsResponse>>;

export declare type ListUserIDsOptions = {
    /**
     * Page to fetch.
     */
    readonly page?: number;
    /**
     * Number of users to retrieve per page.
     */
    readonly hitsPerPage?: number;
};

export declare type ListUserIDsResponse = {
    /**
     * List of users id
     */
    readonly userIDs: readonly UserIDResponse[];
};

export declare type Log = {
    /** Timestamp in ISO-8601 format. */
    readonly timeStamp: string;
    /** Rest type of the method. */
    readonly method: string;
    /** Http response code. */
    readonly answerCode: string;
    /** Request body. It’s truncated after 1000 characters. */
    readonly queryBody: string;
    /** Answer body. It’s truncated after 1000 characters. */
    readonly answer: string;
    /** Request URL. */
    readonly url: string;
    /** Client ip of the call. */
    readonly ip: string;
    /** SHA1 ID of entry. */
    readonly sha1: string;
    /** Request Headers (API Key is obfuscated). */
    readonly queryHeaders: string;
    /** Number Of Api Calls */
    readonly numberOfApiCalls: string;
    /** Processing time for the query. This does not include network time. */
    readonly processingTimeMs: string;
    /** Number of hits returned for the query. */
    readonly numberOfQueryHits: string;
    /** Exhaustive flags used during the query. */
    readonly exhaustive?: boolean;
    /** Index name of the log */
    readonly index: string;
};

export declare const moveIndex: (base: SearchClient) => (from: string, to: string, requestOptions?: RequestOptions | undefined) => Readonly<WaitablePromise<IndexOperationResponse>>;

export declare const multipleBatch: (base: SearchClient) => (requests: readonly BatchRequest[], requestOptions?: RequestOptions | undefined) => Readonly<WaitablePromise<MultipleBatchResponse>>;

export declare type MultipleBatchResponse = {
    readonly objectIDs: readonly string[];
    readonly taskID: {
        readonly [key: string]: number;
    };
};

export declare type MultipleGetObject = {
    readonly indexName: string;
    readonly objectID: string;
    readonly attributesToRetrieve?: readonly string[];
};

export declare const multipleGetObjects: (base: SearchClient) => <TObject>(requests: readonly MultipleGetObject[], requestOptions?: RequestOptions | undefined) => Readonly<Promise<MultipleGetObjectsResponse<TObject>>>;

export declare type MultipleGetObjectsResponse<TResult> = {
    readonly results: readonly TResult[];
};

export declare const multipleQueries: (base: SearchClient) => <TObject>(queries: readonly MultipleQueriesQuery[], requestOptions?: (RequestOptions & MultipleQueriesOptions) | undefined) => Readonly<Promise<MultipleQueriesResponse<TObject>>>;

export declare type MultipleQueriesOptions = {
    readonly strategy?: string;
};

export declare type MultipleQueriesQuery = {
    readonly indexName: string;
    readonly params: SearchOptions;
    readonly query?: string;
};

export declare type MultipleQueriesResponse<TObject> = {
    readonly results: ReadonlyArray<SearchResponse<TObject & ObjectWithObjectID>>;
};

export declare const multipleSearchForFacetValues: (base: SearchClient) => (queries: readonly {
    readonly indexName: string;
    readonly params: SearchForFacetValuesQueryParams & SearchOptions;
}[], requestOptions?: RequestOptions | undefined) => Readonly<Promise<readonly SearchForFacetValuesResponse[]>>;

export declare type ObjectWithObjectID = {
    readonly objectID: string;
};

export declare const partialUpdateObject: (base: SearchIndex) => (object: object, requestOptions?: (RequestOptions & import("../..").ChunkOptions & {
    readonly createIfNotExists?: boolean | undefined;
}) | undefined) => Readonly<WaitablePromise<PartialUpdateObjectResponse>>;

export declare type PartialUpdateObjectResponse = {
    readonly taskID: number;
    readonly objectID: string;
};

export declare const partialUpdateObjects: (base: SearchIndex) => (objects: readonly Record<string, any>[], requestOptions?: (RequestOptions & import("../..").ChunkOptions & {
    readonly createIfNotExists?: boolean | undefined;
}) | undefined) => Readonly<WaitablePromise<readonly BatchResponse[]>>;

export declare type PartialUpdateObjectsOptions = ChunkOptions & {
    readonly createIfNotExists?: boolean;
};

export declare type PersonalizationStrategy = {
    readonly eventsScoring: {
        readonly [key: string]: EventScoring;
    };
    readonly facetsScoring: {
        readonly [key: string]: FacetScoring;
    };
};

export declare const removeUserID: (base: SearchClient) => (userID: string, requestOptions?: RequestOptions | undefined) => Readonly<Promise<RemoveUserIDResponse>>;

export declare type RemoveUserIDResponse = {
    /**
     * Date of deletion
     */
    readonly deletedAt: string;
};

export declare const replaceAllObjects: (base: SearchIndex) => (objects: readonly object[], requestOptions?: (ReplaceAllObjectsOptions & RequestOptions) | undefined) => Readonly<WaitablePromise<void>>;

export declare type ReplaceAllObjectsOptions = {
    readonly safe?: boolean;
};

export declare const replaceAllRules: (base: SearchIndex) => (rules: readonly Rule[], requestOptions?: (RequestOptions & SaveRulesOptions) | undefined) => Readonly<WaitablePromise<SaveRulesResponse>>;

export declare const replaceAllSynonyms: (base: SearchIndex) => (synonyms: readonly Synonym[], requestOptions?: RequestOptions | undefined) => Readonly<WaitablePromise<SaveSynonymsResponse>>;

export declare const restoreApiKey: (base: SearchClient) => (apiKey: string, requestOptions?: RequestOptions | undefined) => Readonly<WaitablePromise<RestoreApiKeyResponse>>;

export declare type RestoreApiKeyResponse = {
    /**
     * Restoration date of the API key.
     */
    readonly createdAt: string;
};

export declare type Rule = {
    /** Unique identifier for the rule (format: [A-Za-z0-9_-]+). */
    readonly objectID: string;
    /** Condition of the rule, expressed using the following variables: pattern, anchoring, context. */
    readonly condition?: Condition;
    /** Consequence of the rule. At least one of the following object must be used: params, promote, hide, userData */
    readonly consequence?: Consequence;
    /** This field is intended for rule management purposes, in particular to ease searching for rules and presenting them to human readers. It is not interpreted by the API. */
    readonly description?: string;
    /** Whether the rule is enabled. Disabled rules remain in the index, but are not applied at query time. */
    readonly enabled?: boolean;
    /**
     * By default, rules are permanently valid. When validity periods are specified, the rule applies only during those periods; it is ignored the rest of the time.
     * The list must not be empty.
     */
    readonly validity?: readonly TimeRange[];
};

export declare const saveObject: (base: SearchIndex) => (object: object, requestOptions?: (RequestOptions & import("../..").ChunkOptions & {
    readonly autoGenerateObjectIDIfNotExist?: boolean | undefined;
}) | undefined) => Readonly<WaitablePromise<SaveObjectResponse>>;

export declare type SaveObjectResponse = {
    readonly taskID: number;
    readonly objectID: string;
};

export declare const saveObjects: (base: SearchIndex) => (objects: readonly Record<string, any>[], requestOptions?: (RequestOptions & import("../..").ChunkOptions & {
    readonly autoGenerateObjectIDIfNotExist?: boolean | undefined;
}) | undefined) => Readonly<WaitablePromise<readonly BatchResponse[]>>;

export declare type SaveObjectsOptions = ChunkOptions & {
    readonly autoGenerateObjectIDIfNotExist?: boolean;
};

export declare const saveRule: (base: SearchIndex) => (rule: Rule, requestOptions?: RequestOptions | undefined) => Readonly<WaitablePromise<SaveRuleResponse>>;

export declare type SaveRuleResponse = {
    readonly updatedAt: number;
    readonly taskID: number;
};

export declare const saveRules: (base: SearchIndex) => (rules: readonly Rule[], requestOptions?: (RequestOptions & SaveRulesOptions) | undefined) => Readonly<WaitablePromise<SaveRulesResponse>>;

export declare type SaveRulesOptions = {
    readonly forwardToReplicas?: boolean;
    readonly clearExistingRules?: boolean;
};

export declare type SaveRulesResponse = {
    readonly updatedAt: number;
    readonly taskID: number;
};

export declare const saveSynonym: (base: SearchIndex) => (synonym: Synonym, requestOptions?: (RequestOptions & SaveSynonymsOptions) | undefined) => Readonly<WaitablePromise<SaveSynonymsResponse>>;

export declare const saveSynonyms: (base: SearchIndex) => (synonyms: readonly Synonym[], requestOptions?: (SaveSynonymsOptions & RequestOptions) | undefined) => Readonly<WaitablePromise<SaveSynonymsResponse>>;

export declare type SaveSynonymsOptions = {
    readonly forwardToReplicas?: boolean;
    readonly replaceExistingSynonyms?: boolean;
};

export declare type SaveSynonymsResponse = {
    readonly updatedAt: number;
    readonly taskID: number;
};

export declare const ScopeEnum: {
    readonly [key: string]: ScopeType;
};

export declare type ScopeType = 'settings' | 'synonyms' | 'rules';

export declare const search: (base: SearchIndex) => <TObject>(query: string, requestOptions?: (RequestOptions & SearchOptions) | undefined) => Readonly<Promise<SearchResponse<TObject>>>;

export declare type SearchClient = {
    readonly appId: string;
    readonly transporter: Transporter;
    readonly addAlgoliaAgent: (segment: string, version?: string) => void;
};

export declare type SearchClientOptions = {
    readonly appId: string;
    readonly apiKey: string;
    readonly authMode?: AuthModeType;
};

export declare const searchForFacetValues: (base: SearchIndex) => (facetName: string, facetQuery: string, requestOptions?: (RequestOptions & SearchOptions) | undefined) => Readonly<Promise<SearchForFacetValuesResponse>>;

export declare type SearchForFacetValuesQueryParams = {
    readonly facetName: string;
    readonly facetQuery: string;
};

export declare type SearchForFacetValuesResponse = {
    readonly facetHits: readonly FacetHit[];
    readonly exhaustiveFacetsCount: boolean;
    readonly processingTimeMS?: number;
};

export declare type SearchIndex = {
    readonly appId: string;
    readonly indexName: string;
    readonly transporter: Transporter;
};

export declare type SearchOptions = {
    readonly query?: string;
    readonly similarQuery?: string;
    readonly facetFilters?: ReadonlyArray<readonly string[]>;
    readonly optionalFilters?: ReadonlyArray<readonly string[]>;
    readonly numericFilters?: ReadonlyArray<readonly string[]>;
    readonly tagFilters?: ReadonlyArray<readonly string[]>;
    readonly sumOrFiltersScores?: boolean;
    readonly filters?: string;
    readonly page?: number;
    readonly hitsPerPage?: number;
    readonly offset?: number;
    readonly length?: number;
    readonly attributesToHighlight?: readonly string[];
    readonly attributesToSnippet?: readonly string[];
    readonly attributesToRetrieve?: readonly string[];
    readonly highlightPreTag?: string;
    readonly highlightPostTag?: string;
    readonly snippetEllipsisText?: string;
    readonly restrictHighlightAndSnippetArrays?: boolean;
    readonly facets?: readonly string[];
    readonly maxValuesPerFacet?: number;
    readonly facetingAfterDistinct?: boolean;
    readonly minWordSizefor1Typo?: number;
    readonly minWordSizefor2Typos?: number;
    readonly allowTyposOnNumericTokens?: boolean;
    readonly disableTypoToleranceOnAttributes?: readonly string[];
    readonly queryType?: string;
    readonly removeWordsIfNoResults?: string;
    readonly advancedSyntax?: boolean;
    readonly advancedSyntaxFeatures?: readonly string[];
    readonly optionalWords?: readonly string[];
    readonly disableExactOnAttributes?: readonly string[];
    readonly exactOnSingleWordQuery?: string;
    readonly alternativesAsExact?: readonly string[];
    readonly enableRules?: boolean;
    readonly ruleContexts?: readonly string[];
    readonly distinct?: number;
    readonly analytics?: boolean;
    readonly analyticsTags?: readonly string[];
    readonly synonyms?: boolean;
    readonly replaceSynonymsInHighlight?: boolean;
    readonly minProximity?: number;
    readonly responseFields?: readonly string[];
    readonly maxFacetHits?: number;
    readonly percentileComputation?: boolean;
    readonly clickAnalytics?: boolean;
    readonly personalizationImpact?: number;
    readonly enablePersonalization?: boolean;
    readonly restrictSearchableAttributes?: readonly string[];
    readonly sortFacetValuesBy?: string;
    readonly typoTolerance?: any;
    readonly aroundLatLng?: string;
    readonly aroundLatLngViaIP?: boolean;
    readonly aroundRadius?: any;
    readonly aroundPrecision?: number;
    readonly minimumAroundRadius?: number;
    readonly insideBoundingBox?: ReadonlyArray<readonly number[]>;
    readonly insidePolygon?: ReadonlyArray<readonly number[]>;
    readonly ignorePlurals?: any;
    readonly removeStopWords?: readonly string[];
    readonly getRankingInfo?: boolean;
};

export declare type SearchResponse<TObject = {}> = {
    readonly hits: ReadonlyArray<TObject & ObjectWithObjectID>;
    readonly page: number;
    readonly length?: number;
    readonly offset?: number;
    readonly nbHits: number;
    readonly nbPages: number;
    readonly hitsPerPage: number;
    readonly processingTimeMs: number;
    readonly exhaustiveNbHits: boolean;
    readonly exhaustiveFacetsCount?: boolean;
    readonly facets?: {
        readonly [key: string]: {
            readonly [key: string]: number;
        };
    };
    readonly facetsStats?: {
        readonly [key: string]: {
            readonly min: number;
            readonly max: number;
            readonly avg: number;
            readonly sum: number;
        };
    };
    readonly query: string;
    readonly queryAfterRemoval?: string;
    readonly params: string;
    readonly queryID?: string;
    readonly message?: string;
    readonly aroundLatLng?: string;
    readonly automaticRadius?: string;
    readonly serverUsed?: string;
    readonly index?: string;
    readonly indexUsed?: string;
    readonly abTestVariantID?: number;
    readonly parsedQuery?: string;
    readonly userData?: any;
    readonly appliedRules?: ReadonlyArray<{
        readonly [key: string]: any;
    }>;
};

export declare const searchRules: (base: SearchIndex) => (query: string, requestOptions?: (RequestOptions & SearchRulesOptions) | undefined) => Readonly<Promise<SearchResponse<Rule>>>;

export declare type SearchRulesOptions = {
    /** Full text query. */
    readonly query?: string;
    /** When specified, restricts matches to rules with a specific anchoring type. When omitted, all anchoring types may match. */
    readonly anchoring?: string;
    /** Restricts matches to contextual rules with a specific context (exact match). */
    readonly context?: string;
    /** Requested page (zero-based). */
    readonly page?: number;
    /** Maximum number of hits in a page. Minimum is 1, maximum is 1000. */
    readonly hitsPerPage?: number;
    /**
     * When specified, restricts matches to rules with a specific enabled status.
     * When absent (default), all rules are retrieved, regardless of their enabled status.
     */
    readonly enabled?: boolean;
};

export declare const searchSynonyms: (base: SearchIndex) => (query: string, requestOptions?: (SearchSynonymsOptions & RequestOptions) | undefined) => Readonly<Promise<SearchSynonymsResponse>>;

export declare type SearchSynonymsOptions = {
    readonly query?: string;
    readonly type?: SynonymType;
    readonly page?: number;
    readonly hitsPerPage?: number;
};

export declare type SearchSynonymsResponse = {
    readonly hits: readonly Synonym[];
    readonly nbHits: number;
};

export declare const searchUserIDs: (base: SearchClient) => (query: string, requestOptions?: (SearchUserIDsOptions & RequestOptions) | undefined) => Readonly<Promise<SearchUserIDsResponse>>;

export declare type SearchUserIDsOptions = {
    /**
     * If specified, only clusters assigned to this cluster can be returned.
     * */
    readonly cluster?: string;
    /**
     * Page to fetch.
     */
    readonly page?: number;
    /**
     * Number of users to return by page.
     */
    readonly hitsPerPage?: number;
};

export declare type SearchUserIDsResponse = {
    /**
     * List of userID matching the query.
     */
    readonly hits: readonly UserIDResponse[];
    /**
     * Current page.
     */
    readonly page: number;
    /**
     * Number of userIDs matching the query.
     */
    readonly nbHits: number;
    /**
     * Number of hits retrieved per page.
     */
    readonly hitsPerPage: number;
    /**
     * Timestamp of the last update of the index.
     */
    readonly updatedAt: number;
};

export declare type SecuredApiKeyRestrictions = SearchOptions & {
    /**
     * A Unix timestamp used to define the expiration date of the API key.
     */
    readonly validUntil?: number;
    /**
     * List of index names that can be queried.
     */
    readonly restrictIndices?: readonly string[] | string;
    /**
     * IPv4 network allowed to use the generated key. This is used for more protection against API key leaking and reuse.
     */
    readonly restrictSources?: string;
    /**
     * Specify a user identifier. This is often used with rate limits.
     */
    readonly userToken?: string;
};

export declare const setPersonalizationStrategy: (base: SearchClient) => (personalizationStrategy: PersonalizationStrategy, requestOptions?: RequestOptions | undefined) => Readonly<Promise<SetPersonalizationStrategyResponse>>;

export declare type SetPersonalizationStrategyResponse = {
    readonly updatedAt: number;
};

export declare const setSettings: (base: SearchIndex) => (settings: IndexSettings, requestOptions?: RequestOptions | undefined) => Readonly<WaitablePromise<SetSettingsResponse>>;

export declare type SetSettingsResponse = {
    readonly taskID: number;
    readonly UpdatedAt: number;
};

export declare const StrategyEnum: {
    readonly [key: string]: StrategyType;
};

export declare type StrategyType = 'none' | 'stopIfEnoughMatches';

export declare type Synonym = {
    readonly objectID: string;
    readonly type: SynonymType;
    readonly synonyms?: readonly string[];
    readonly input?: string;
    readonly word?: string;
    readonly corrections?: readonly string[];
    readonly placeholder?: string;
    readonly replacements?: readonly string[];
};

export declare const SynonymEnum: {
    readonly [key: string]: SynonymType;
};

export declare type SynonymType = 'synonym' | 'oneWaySynonym' | 'altCorrection1' | 'altCorrection2' | 'placeholder';

export declare type TaskStatusResponse = {
    readonly status: string;
    readonly pendingTask: boolean;
};

export declare type TimeRange = {
    /** DateTime with UTC offset for Serialization/Deserialization in unix timespam */
    readonly from: number;
    /** DateTime with UTC offset for Serialization/Deserialization in unix timespam */
    readonly until: number;
};

export declare const updateApiKey: (base: SearchClient) => (apiKey: string, requestOptions?: (import("../..").AddApiKeyOptions & Pick<RequestOptions, string | number>) | undefined) => Readonly<WaitablePromise<UpdateApiKeyResponse>>;

export declare type UpdateApiKeyOptions = AddApiKeyOptions;

export declare type UpdateApiKeyResponse = {
    /**
     * The api key.
     */
    readonly key: string;
    /**
     * Date of update
     */
    readonly updatedAt: string;
};

export declare type UserIDResponse = {
    /**
     * userID of the user.
     */
    readonly userID: string;
    /**
     * Cluster on which the user is assigned
     */
    readonly clusterName: string;
    /**
     * Number of records belonging to the user.
     */
    readonly nbRecords: number;
    /**
     * Data size used by the user.
     */
    readonly dataSize: number;
};

export declare const waitTask: (base: SearchIndex) => (taskID: number, requestOptions?: RequestOptions | undefined) => Readonly<Promise<void>>;

export { }
